pub struct Multiplexer { /* private fields */ }Expand description
The Boolean multiplexer of Koza (1992): k address bits select one of 2^k data bits, which
is the output. The 11-multiplexer (k = 3, 2048 cases) is Koza’s; the 6-multiplexer (k =
2) is a smaller version.
Its primitives are Koza’s: the functions and, or, not and if (of three arguments),
and the inputs a0 to a{k-1} (the address, a0 its least significant bit), then d0 to
d{2^k-1} (the data). One type: the untyped genetic programming of the paper.
use genoxide::gp::boolean::Multiplexer;
let problem = Multiplexer::new(3)?; // Koza's 11-multiplexer
assert_eq!((problem.inputs(), problem.cases()), (11, 2048));
let set = problem.primitives();
// the right output everywhere: the data bit that the address selects
let tree = set.parse(
"if(a2, if(a1, if(a0, d7, d6), if(a0, d5, d4)), if(a1, if(a0, d3, d2), if(a0, d1, d0)))",
)?;
assert_eq!(problem.errors(&tree), 0);
// one data bit is right in the cases that select it, and in half the others
assert_eq!(problem.errors(&set.parse("d0")?), 896);Implementations
Sourceimpl Multiplexer
impl Multiplexer
Sourcepub fn new(address_bits: usize) -> Result<Self>
pub fn new(address_bits: usize) -> Result<Self>
The multiplexer of address_bits address bits, 1 to 4: the 3-, 6-, 11- and
20-multiplexer.
Errors
Error::InvalidSetting (setting address_bits) for 0 or more than 4 address bits.
Sourcepub fn address_bits(&self) -> usize
pub fn address_bits(&self) -> usize
The number of address bits.
Sourceimpl Multiplexer
impl Multiplexer
Sourcepub fn primitives(&self) -> &PrimitiveSet<Logic>
pub fn primitives(&self) -> &PrimitiveSet<Logic>
Sourcepub fn cases(&self) -> u64
pub fn cases(&self) -> u64
The number of cases of the truth table, 2^inputs. In case c, input i (in the
order of the set’s terminals) is bit i of c.
Sourcepub fn targets(&self) -> &[u64]
pub fn targets(&self) -> &[u64]
The right outputs, 64 cases per word: case c at bit c % 64 of word c / 64.